#ifndef __J1708_H__
#define __J1708_H__

// Required by some macro definitions below.
#include <windows.h>

#ifndef CTL_CODE
    #include "winioctl.h"
#endif

//--------------------------------------------------------------------------------------------------
// J1708 Macros
//--------------------------------------------------------------------------------------------------
#define J1708_MID_CHAR_SIZE             1
#define J1708_MAX_DATA_SIZE             19
#define J1708_PKT_STS_SIZE              1
#define J1708_MAX_PACKET_SIZE           (J1708_MID_CHAR_SIZE + J1708_MAX_DATA_SIZE + J1708_PKT_STS_SIZE)

#define J1708_PKT_STS_BYTE_INDEX        (J1708_MAX_PACKET_SIZE - 1)
#define J1708_PKT_STS_SIZE_BIT_MASK     0x1F
#define J1708_PKT_STS_OVERFLOW_BIT      0x40
#define J1708_PKT_STS_CHECKSUM_BIT      0x80

#define J1708_MAX_XMT_PRIORITY          8

// Define IOCTL's.
//
#define IOCTL_CUST_BASE 2048
#define IOCTL_J1708_DIS_RTL_MSG             CTL_CODE(FILE_DEVICE_SERIAL_PORT, IOCTL_CUST_BASE + 0, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_J1708_ENA_RTL_MSG             CTL_CODE(FILE_DEVICE_SERIAL_PORT, IOCTL_CUST_BASE + 1, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_J1708_GET_RCV_PKT_EVT_NAME    CTL_CODE(FILE_DEVICE_SERIAL_PORT, IOCTL_CUST_BASE + 2, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_J1708_GET_DRIVER_STATUS       CTL_CODE(FILE_DEVICE_SERIAL_PORT, IOCTL_CUST_BASE + 3, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_J1708_GET_XMT_PRIORITY        CTL_CODE(FILE_DEVICE_SERIAL_PORT, IOCTL_CUST_BASE + 4, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_J1708_SET_XMT_PRIORITY        CTL_CODE(FILE_DEVICE_SERIAL_PORT, IOCTL_CUST_BASE + 5, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_J1708_GET_MLT_PACKETS         CTL_CODE(FILE_DEVICE_SERIAL_PORT, IOCTL_CUST_BASE + 6, METHOD_BUFFERED, FILE_ANY_ACCESS)

// Define driver error codes.
//
#define APPL_ERROR_BIT  29
#define J1708_ERROR_BIT 28
#define J1708_ERROR_CODE(errorNum) ( (0x01 << APPL_ERROR_BIT) | (0x01 << J1708_ERROR_BIT) | (errorNum) )
//
#define J1708_ERROR_BAD_CHECKSUM            J1708_ERROR_CODE(1)
#define J1708_ERROR_BUFFER_OVERFLOW         J1708_ERROR_CODE(2)
#define J1708_ERROR_PACKET_DROPPED          J1708_ERROR_CODE(3)
#define J1708_ERROR_DRIVER_ALREADY_OPEN     J1708_ERROR_CODE(4)
#define J1708_ERROR_CANNOT_INIT_INTRPT      J1708_ERROR_CODE(5)
#define J1708_ERROR_TX_PACKET_TOO_BIG       J1708_ERROR_CODE(6)
#define J1708_ERROR_CANNOT_ENABLE_UART      J1708_ERROR_CODE(7)
#define J1708_ERROR_RX_BUFFER_TOO_SMALL     J1708_ERROR_CODE(8)
#define J1708_ERROR_RX_ZERO_PACKET_COUNT    J1708_ERROR_CODE(9)

//--------------------------------------------------------------------------------------------------
// J1708 Types
//--------------------------------------------------------------------------------------------------
typedef struct
{
    DWORD sts1;
    DWORD sts2;
    DWORD sts3;
    DWORD sts4;
    DWORD sts5;
}
    J1708_DRVR_STS_STRUCT, * PJ1708_DRVR_STS_STRUCT;

//--------------------------------------------------------------------------------------------------
// J1708 Inline Functions
//--------------------------------------------------------------------------------------------------
DWORD __inline J1708ReadMultiplePackets(HANDLE hPort, BYTE (*pPacketBuffers)[J1708_MAX_PACKET_SIZE], DWORD dwNumOfPackets)
{
    DWORD dwPacketsRead;
    DeviceIoControl(hPort, IOCTL_J1708_GET_MLT_PACKETS, NULL, 0, (LPVOID) pPacketBuffers, dwNumOfPackets, &dwPacketsRead, NULL);
    return(dwPacketsRead);
}

//--------------------------------------------------------------------------------------------------
#endif // __J1708_H__
